package ru.landgrafhomyak.db.skeleton1.api.errors.definition

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.ReflectionApi
import ru.landgrafhomyak.db.skeleton1.api.errors.SchemaDefinitionException

@ReflectionApi
@OptIn(ExperimentalCoroutinesApi::class)
public class ExportedEntitiesNotImportedException : SchemaDefinitionException, CopyableThrowable<ExportedEntitiesNotImportedException> {
	public val sourceScope: Any
	public val entities: Collection<Any>

	public constructor(sourceScope: Any, entities: Collection<Any>) : super(formatMessage(sourceScope, entities)) {
		this.sourceScope = sourceScope
		this.entities = entities
	}

	public constructor(sourceScope: Any, entities: Collection<Any>, customMessage: String) : super(customMessage) {
		this.sourceScope = sourceScope
		this.entities = entities
	}

	public constructor(sourceScope: Any, entities: Collection<Any>, customMessage: String, cause: Throwable) : super(customMessage, cause) {
		this.sourceScope = sourceScope
		this.entities = entities
	}

	public constructor(sourceScope: Any, entities: Collection<Any>, cause: Throwable) : super(formatMessage(sourceScope, entities), cause) {
		this.sourceScope = sourceScope
		this.entities = entities
	}

	public constructor(
		sourceScope: Any, entities: Collection<Any>,
		message: String?, cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause, _marker) {
		this.sourceScope = sourceScope
		this.entities = entities
	}

	override fun createCopy(): ExportedEntitiesNotImportedException? {
		return ExportedEntitiesNotImportedException(
			sourceScope = this.sourceScope,
			entities = this.entities,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(
			sourceScope: Any, entities: Collection<Any>,
		): String = "Entities exported from $sourceScope wasn't imported to any other scope: ${entities.joinToString()}"
	}
}