package ru.landgrafhomyak.db.skeleton1.api.errors.definition

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.ReflectionApi
import ru.landgrafhomyak.db.skeleton1.api.errors.SchemaDefinitionException
import ru.landgrafhomyak.db.skeleton1.api.table.Table

@ReflectionApi
@OptIn(ExperimentalCoroutinesApi::class)
public class TableWithoutColumnsException : SchemaDefinitionException, CopyableThrowable<TableWithoutColumnsException> {
	public val table: Table<*, *>

	public constructor(table: Table<*, *>) : super(formatMessage(table)) {
		this.table = table
	}

	public constructor(table: Table<*, *>, customMessage: String) : super(customMessage) {
		this.table = table
	}

	public constructor(table: Table<*, *>, customMessage: String, cause: Throwable) : super(customMessage, cause) {
		this.table = table
	}

	public constructor(table: Table<*, *>, cause: Throwable) : super(formatMessage(table), cause) {
		this.table = table
	}

	public constructor(
		table: Table<*, *>,
		message: String?, cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause, _marker) {
		this.table = table
	}

	override fun createCopy(): TableWithoutColumnsException? {
		return TableWithoutColumnsException(
			table = this.table,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(
			table: Table<*, *>,
		): String = "Table $table doesn't have columns"
	}
}