package ru.landgrafhomyak.db.skeleton1.api.module

import ru.landgrafhomyak.db.skeleton1.api.LowLevelApi
import ru.landgrafhomyak.db.skeleton1.api.statement.RawStatement
import ru.landgrafhomyak.db.skeleton1.api.statement._Statement
import ru.landgrafhomyak.db.skeleton1.api.runtime.InputRow
import ru.landgrafhomyak.db.skeleton1.api.table.Table


public interface ModuleUpgradeScope<nmUE : Any, omUE : Any> : ModuleConstructorScope<nmUE> {
	// Table -------------------------------------------------------------------
	public fun <tUE : Any> keepTable(table: Table<tUE, omUE>): Table<tUE, nmUE>
	public fun <tUE : Any> keepAndRenameTable(
		table: Table<tUE, omUE>,
		newNamespace: Namespace<nmUE>? = null, newName: String
	): Table<tUE, nmUE>

	public fun <ntUE : Any, otUE : Any> upgradeTable(
		oldTable: Table<otUE, omUE>,
		upgrade: Table.Upgrade<ntUE, otUE>
	): Table<ntUE, nmUE>

	public fun <ntUE : Any, otUE : Any> upgradeAndRenameTable(
		table: Table<otUE, omUE>,
		newNamespace: Namespace<nmUE>? = null, newName: String,
		upgrade: Table.Upgrade<ntUE, otUE>
	): Table<ntUE, nmUE>

	public fun deleteTable(table: Table<*, omUE>)
	public fun <tUE : Any> deleteTableAfterModuleUpgrade(table: Table<tUE, omUE>): Table<tUE, nmUE>

	public fun exportTable(table: Table<*, omUE>)
	public fun <tUE : Any> importTable(
		table: Table<tUE, *>,
		namespace: Namespace<nmUE> = this.rootNs, name: String
	): Table<tUE, nmUE>

	public fun <ntUE : Any, otUE : Any> importAndUpgradeTable(
		table: Table<otUE, *>,
		namespace: Namespace<nmUE> = this.rootNs, name: String,
		upgrade: Table.Upgrade<ntUE, otUE>
	): Table<ntUE, nmUE>

	// Object ------------------------------------------------------------------

	public fun <oUE : Any?> keepModuleScopedObject(obj: ModuleScopedObject<oUE, omUE>): ModuleScopedObject<oUE, nmUE>
	public fun <oUE : Any?> renameModuleScopedObject(
		obj: ModuleScopedObject<oUE, omUE>,
		newNamespace: Namespace<nmUE>? = null, newName: String
	): ModuleScopedObject<oUE, nmUE>

	public fun <oUE : Any?> deleteModuleScopedObject(obj: ModuleScopedObject<oUE, omUE>)
	public fun <oUE : Any?> deleteModuleScopedObjectAfterModuleUpgrade(obj: ModuleScopedObject<oUE, omUE>): ModuleScopedObject<oUE, nmUE>

	public fun exportModuleScopedObject(obj: ModuleScopedObject<*, omUE>)
	public fun <oUE : Any> importModuleScopedObject(
		obj: ModuleScopedObject<oUE, *>,
		namespace: Namespace<nmUE> = this.rootNs, name: String
	): ModuleScopedObject<oUE, nmUE>


	// Module ------------------------------------------------------------------

	public fun <smUE : Any> keepModule(module: Module<smUE>): Module<smUE>
	public fun <smUE : Any> keepAndMoveModule(oldModule: Module<smUE>, newNs: Namespace<nmUE>): Module<smUE>

	public fun <smUE : Any> upgradeModule(oldModule: Module<*>, template: Module.Template<smUE>): Module<smUE>
	public fun <smUE : Any> upgradeAndMoveModule(
		oldModule: Module<*>,
		newNamespace: Namespace<nmUE>,
		template: Module.Template<smUE>
	): Module<smUE>

	public fun deleteModule(oldModule: Module<*>)
	public fun <smUE : Any> deleteModuleAfterUpgrade(oldModule: Module<smUE>): Module<smUE>

	public fun exportModule(table: Module<*>)

	public fun <smUE : Any> importModule(
		module: Module<smUE>,
		namespace: Namespace<nmUE>,
	): Module<smUE>

	public fun <nsmUE : Any, osmUE : Any> importAndUpgradeModule(
		module: Module<osmUE>,
		namespace: Namespace<nmUE>,
		upgrade: Module.Upgrade<nsmUE, osmUE>
	): Module<nsmUE>

	// Statements --------------------------------------------------------------

	public fun addStatementAfterUpgrade(stmt: _Statement.Void2Void<*>)
	public fun addStatementAfterUpgrade(stmt: RawStatement.Void2Void.ImplementationsProvider<*>)

	@LowLevelApi
	public fun <sUE : Any> _addStatementAfterUpgrade(stmt: _Statement.Params2Void<sUE>): InputRow._Scope<sUE, Unit>

	@LowLevelApi
	public fun <sUE : Any> _addStatementAfterUpgrade(stmt: RawStatement.Params2Void.ImplementationsProvider<sUE>): InputRow._Scope<sUE, Unit>
}